/* 
 * Copyright (C) 2006, iRex Technologies B.V.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef PDFVIEW_H_
#define PDFVIEW_H_

#define Object XtObject
#include <Xm/XmAll.h>
#undef Object
#include <liberdm/erdm.h>
#include "poppler/Link.h"
#include "PageInfo.h"
struct rectangle;
struct Controller;
class CPageInfo;

//////////////////////////////////////////////////////////////////////////
// pdf view class
// 1. display page
// 2. interact with user
//////////////////////////////////////////////////////////////////////////
class CPDFView
{
private:
    Controller *    ctrl;

    // display
    Widget          drawArea;   // install a callback function
    
    // zoom and pan
    rectangle       zoomRect;   // used by zoom in/out 
    point           panPt;      // by pan
    GBool           isSmallOffset(const int x, const int y);
    void            adjustZoomRect(const int x, const int y);
    void            adjustPosition(const rectangle & oldRect, double ratio);
    double          calRatio(rectangle & rect, GBool bZoomin = gTrue);
    

    // hyper link
    GBool           doLink(int x, int y);
    GBool           doAction(LinkAction * action);
    void            cvtDevToUser(CPageInfo *page, const int x, const int y, double & ux, double & uy);
    
    // password
    GBool           bAuth;
    Widget          passwordDialog;
    Widget          passwordText;
    GooString *     password;
    int             dialogDone;
    void            initPasswordDialog();
    static void     passwordTextVerifyCbk(Widget widget, XtPointer ptr,XtPointer callData);
    static void     passwordOkCbk(Widget widget, XtPointer ptr,XtPointer callData);
    static void     passwordCancelCbk(Widget widget, XtPointer ptr,XtPointer callData);

    // items selected to pre-render
    void            issueNewRequests();
    void            issueItem(const int pn, GBool bHead = gTrue);
    void            issuePrevNextPages(const int offset);
    void            issueLinks(CPageInfo *page);
    void            issueLink(Link * p);
    int             getLinkPage(LinkAction *action);

    // for A4 mode
    GBool           isA4Mode(); 
    void            startA4Reading();
    void            A4GoPrev();
    void            A4GoNext();
    
public:
    CPDFView(Widget wnd, Controller * c );
    ~CPDFView(void);
    
    void rotateCurrentPage(const int angle);    // clock wise + 
    void drawPage(CPageInfo * page);
    void displayPage(const int pageNumber);     // check and draw
    void onPageRendered(const int pageNumber);  // receive message
    void zoomFromRect(int left, int top, int right, int bottom);
    void zoomToRect(int left, int top, int right, int bottom);
    void updateToolbar(CPageInfo *page);
    void keyPressCbk(XmDrawingAreaCallbackStruct *data);
    void refreshDeviceScreen(eDmQuality quality);
    GooString * getPassword();
    static void inputCbk(Widget widget, XtPointer ptr, XtPointer callData);
    static void ipcCbk(XtPointer client_data, int * fd, XtInputId * id);
    static void redrawCbk(Widget widget, XtPointer ptr, XtPointer callData);
    static void motionCbk(Widget w, XtPointer cdata, XEvent *event, Boolean *cont);
};

#endif
