/* 
 * Copyright (C) 2006, iRex Technologies B.V.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef PAGEINFO_H_
#define PAGEINFO_H_

#define Object XtObject
#include <Xm/XmAll.h>
#undef Object

#include "splash/SplashBitmap.h"
#include "poppler/Object.h"
#include "poppler/SplashOutputDev.h"
#include "goo/GooList.h"
#include "poppler/Link.h"
#include "poppler/TextOutputDev.h"
#include "poppler/Gfx.h"
#include <signal.h>

struct point
{
    int x, y;
};

struct rectangle
{
    int left;
    int top;
    int right;
    int bottom;
};
//////////////////////////////////////////////////////////////////////////
// pageinfo stores all information used to 
// 1. render a pdf page
// 2. display a pdf page
// 3. calculat a pdf page (A4,..., A0)
// 4. something related to state
//////////////////////////////////////////////////////////////////////////
class CPageInfo
{
public:
    CPageInfo(void);
    ~CPageInfo(void);

public:
    double          pageDx;         // getPageCropWidth
    double          pageDy;         // getPageCropHeight
    int             pageRotate;     // getPageRotate
    double          pageZoom;       // in/out different from zoom (global setting)
    double          hDPI;           // out
    double          vDPI;           // out
    int             rotate;         // rotate, global setting
    int             pageNumber;
    int             timestamp;      // pre-rendering

    // render result
    //    XImage *        image;          // to display on screen
    SplashBitmap *  bitmap;         // by SplashOutputDev
    Links *         links;
    TextPage *      text;	
    RenderRet       state;
    double          ctm[6];         // from xpdf for hyperlink
    double          ictm[6];        // from xpdf for hyperlink

public:
    // methods
    void    initialize();
    void    clear();
    GBool   isSame(const int pn, const double zoom, const int rotate);
    double  calZoom();
};
typedef CPageInfo * PageInfoPtr;

//////////////////////////////////////////////////////////////////////////
// page list management. It should be thread-safe
//////////////////////////////////////////////////////////////////////////
class TodoItem;
class PageList
{
private:
    pthread_mutex_t     mutex;  // protect the list
    PageInfoPtr *       list;
    int                 length;
    
    void    initialize();

public:
    PageList();
    ~PageList();
    
    void        reset(const int size);
    void        clear();
    void        addPage(CPageInfo * info);    
    void        removeOldestOne();
    CPageInfo * getPage(const int number);
    CPageInfo * checkPage(const CPageInfo & info);
    CPageInfo * checkPage(const TodoItem * pItem);
    void        dump();     // for debug
};

//////////////////////////////////////////////////////////////////////////
// todo list management. It should be thread-safe
//////////////////////////////////////////////////////////////////////////
class TodoItem
{
public:
    TodoItem();
    ~TodoItem();

public:
    double  zoom;
    int     rotate;
    int     pageNumber;
    int     timeStamp;          // time stamp when request is issued
};

class TodoList
{
private:
    pthread_mutex_t     mutex;  // protect the list
    GooList *           list;

    void    initialize();
    void    removeDup(TodoItem * item); // remove duplicated items by time stamp

public:
    TodoList();
    ~TodoList();
    
    void        clear();
    void        pushItem(TodoItem * item, GBool bHead = gFalse, GBool bRemoveDup = gTrue);    
    TodoItem *  popItem();
};

#endif
