/* 
 * Copyright (C) 2006, iRex Technologies B.V.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef CHANNELS_H_
#define CHANNELS_H_

#include <liberipc/eripc.h>
#include <liberipc/eripcbusyd.h>
#include <liberipc/eripcpagebar.h>
#include <liberipc/eripctoolbar.h>
#include "goo/gtypes.h"

//////////////////////////////////////////////////////////////////////////
// All channels used by iLiad: busy, pagebar and toolbar
//////////////////////////////////////////////////////////////////////////

// toolbar's state could be changed by:
// 1. User click ---- update by using typing waveform
// 2. App update ---- could be comibed, so that we can refresh screen only once

enum tbDirtyReason
{
    ByUserAction = 0,
    ByApp        = 1,
    ByNone       = 2, 
};

#define Portrait_Landscape  90
#define Landscape_Portrait  -90

class CChannels
{
public:
    erClientChannel_t erbusyChannel;
    erClientChannel_t erpagebarChannel;
    erClientChannel_t ertoolbarChannel;
    erServerChannel_t erIpcChannel;

private:        
    // toolbar state     
    int     nPanState;
    int     nZoominState;
    int     nZoomoutState;
    int     nRotateState;
    GBool   tbDirty;    
    tbDirtyReason reason;
    
    // init all
    void initialize();
public:
    CChannels();
    ~CChannels();


    // busy light    
    void    busy_blink();
    void    busy_on();
    void    busy_off();

    // page bar
    void    pagebar_set_count(const int count);
    void    pagebar_set_current_page(const int number);
    
    // toolbar
    GBool   onReceiveToolbarMsg(const int iconID, const int state); 
    void    setPanState(const int state) { nPanState = state; }
    GBool   isPanEnable() const { return iconState_selected == nPanState; }
    int     getPanState() const { return nPanState; }
    void    setZoomIn(const int state) { nZoominState = state; }
    GBool   isZoomInEnable() const { return iconState_selected == nZoominState; }
    int     getZoomInState() const { return nZoominState; }
    void    setZoomOut(const int state) { nZoomoutState = state; }
    GBool   isZoomOutEnable() const { return iconState_selected == nZoomoutState; }
    int     getZoomOutState() const { return nZoomoutState; }
    void    setRotateState(const int state) { nRotateState = state; }
    int     getRotateState() const { return nRotateState; }
    GBool   isToolbarDirty() const { return tbDirty;}    
    tbDirtyReason
            getToolbarDirtyReason() const { return reason; }
    void    setToolbarDirty(GBool b = gTrue){ tbDirty = b;  }
    void    setToolbarDirtyReason(tbDirtyReason r) { reason = r;}
    void    redrawToolbar(GBool bSync = gTrue); 
    void    popupKeyboard(GBool bPopup = gTrue);        
};


#endif
