/* 
 * Copyright (C) 2006, iRex Technologies B.V.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef CONTROLLER_H_
#define CONTROLLER_H_

#include "PageInfo.h"
#include "PDFCore.h"
#include "RenderThread.h"
#include "XMgr.h"
#include "PDFView.h"
#include "RenderThread.h"
#include <sys/time.h>
#include <liberdm/display.h>
#include "channels.h"


#define ZoomPage        -1
#define ZoomWidth       -2
#define DefZoom         125
#define ZoomMax         300     // should be larger or equal to all ZoomMax
#define ZoomMin         25
#define InvalidPage     -1
#define InvalidZoom     -100
#define DefDPI          72
#define RenderNext      1
#define RenderPrev      -1
#define RenderWidth     5
#define JumpWidth       5       // long press
#define RemovePages     4       // if memory is not enough, how many pages should be removed
#define SmallOffset     5

// define max zoom for these pages
#define A0ZoomMax       100
#define A1ZoomMax       100
#define A2ZoomMax       100
#define A3ZoomMax       200
#define A4ZoomMax       250
#define A5ZoomMax       300
#define A6ZoomMax       300
#define A7ZoomMax       300
#define A8ZoomMax       300
#define A9ZoomMax       300

// mode
#define ModeReading     1
#define ModeNewspaper   2       // this two decide pre-render algorithm
#define ModeA4          3       // for A4 pages

//////////////////////////////////////////////////////////////////////////
//
// controller serves as access pointer, from which every component can 
// access all the other components. 
// controller is initialized, maintained and destroyed by CPDFApp class
//
// settings contains all global parameters. 
// NOTICE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// When settings.zoom is larger than value calculated from page type
// We should use the value without changing settings.zoom
//
// we should check zoom max value when
// 1. issues a new request 
// 2. receives a message from render thread ***
// 3. query a page in page list
//
//////////////////////////////////////////////////////////////////////////
struct Settings
{
    double      zoom;
    int         rotate;
    int         screenX;        // all pages start position
    int         screenY;
    int         currentPage;
    int         drawAreaWidth;  // usually it's a constant value
    int         drawAreaHeight; // could be change in run time, for full screen
    int         renderDir;      // pre-render direction
    int         mode;           // could be newspaper or reading

    Settings()
    {
        zoom = ZoomPage; rotate = 0; screenX = screenY = 0;
        currentPage = -1; drawAreaWidth = SCREEN_WIDTH;
        drawAreaHeight = CLIENT_AREA; renderDir = RenderNext; 
        mode = ModeNewspaper;
    }
    
    void load(const char * manifest);
    void save(const char *  manifest);
};

class CPDFApp;
struct Controller
{
private:
    struct timeval   startStamp;
    struct timeval   now;    
    void initTimeStamp()
    {
        gettimeofday(&startStamp, NULL);
    }
public:
    CPDFApp  *  app;        // becomes NULL when quit from app
    CPDFCore *  core;       // same as app
    CPDFView *  view;       // same as app
    PageList    pageList;   // delete only when quit from app
    TodoList    todoList;   // same as pageList
    XMgr        xMgr;       
    Settings    settings;   // global settings
    CRenderThread
                thread;
    ThreadParam param;
    CChannels   channels;   // all channels in iLiad
    int         updateReq;  // used to control update, when it reaches to 0, update screen

    Controller()
    {
        core = NULL; view = NULL; app = NULL;
        param.bQuit = gFalse;
        param.pageList = &pageList;
        param.todoList = &todoList;
        param.ctrl = this;
        updateReq = 0;
        initTimeStamp();
    }

    int getTimeStamp()
    {
        gettimeofday(&now, NULL);
        return (now.tv_sec - startStamp.tv_sec);
    }        
    
    int getHighTimeStamp()
    {
        gettimeofday(&now, NULL);
        return (now.tv_sec - startStamp.tv_sec) * 1000000 +
                (now.tv_usec - startStamp.tv_usec);
    }
};

#endif
