/* 
 * Copyright (C) 2006, iRex Technologies B.V.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * Portions of this code are taken from xpdf and are
 * Copyright 1996-2003 Glyph & Cog, LLC
 */

#include "PDFApp.h"
#include "poppler/Object.h"
#include "config.h"
#include "poppler/GlobalParams.h"
#include "PDFViewerLog.h"
#include "poppler/SecurityHandler.h"

#ifdef LESSTIF_VERSION
#undef XtDisplay
#undef XtScreen
#undef XtWindow
#undef XtParent
#undef XtIsRealized
#endif

CPDFApp::CPDFApp(int * argc, char *argv[])
{
    controller.app = this;
    setupX(argc, argv);
    initialize();
}

CPDFApp::~CPDFApp(void)
{

}

void CPDFApp::clear()
{
    close();
    delete controller.core; controller.core = NULL;
    delete controller.view; controller.view = NULL;
    PV_LOGPRINTF("Going to remove globalParams %p", globalParams);
    delete globalParams; globalParams = NULL;
}

void CPDFApp::initialize()
{
    globalParams = new GlobalParams("");

    // initialize core
    SplashColor     white;
    white[0] = 255;
    white[1] = 255;
    white[2] = 255;
    controller.core = new CPDFCore(splashModeRGB8, 4, gFalse, white);
    controller.core->setController(&controller);
    
    // Initialize controller. notice!!! 
    // All controller parameters should be initialized here

    // start render thread
    controller.thread.start(&controller.param);
}

void CPDFApp::run()
{
    XtAppMainLoop(controller.xMgr.appContext);
}

// it should be the only exit entry, this function could be called
// by viewer and terminate handler
void CPDFApp::quit()
{
    // clear viewer 
    clear();

    // save settings 
    saveSettings();
    
    // quit from main loop
    XtAppSetExitFlag(controller.xMgr.appContext);
}

void CPDFApp::close()
{
    controller.thread.stop();
    PV_LOGPRINTF("Stop Render Thread!");
    controller.core->close();
    PV_LOGPRINTF("Clear PDFCore!");
    controller.pageList.clear();
    PV_LOGPRINTF("Clear page list!");
    controller.todoList.clear(); 
    PV_LOGPRINTF("Clear todo list!");  
}

void CPDFApp::setupX(int * argc, char *argv[])
{
    // setup X
    controller.xMgr.setupX(argc, argv);

    // Initialize view
    controller.view = new CPDFView(controller.xMgr.drawArea, &controller);  
}

// check the page number, zoom factor and rotation
void CPDFApp::loadSettings(GooString * fileName)
{
    // load settings from manifest file
    strncpy(path, fileName->getCString(), MAX_PATH);
    char * p = strrchr(path, '/');
    if (NULL == p) return;

    *++p = 0; strcat(path, "manifest.xml");
    controller.settings.load(path);

    // validate
    PDFDoc * pdfdoc = controller.core->getPDFDoc();    
    if (controller.settings.currentPage <= 0 ||
        controller.settings.currentPage > pdfdoc->getNumPages())
    {
        controller.settings.currentPage = 1;
    }        
    
    if ((controller.settings.zoom > ZoomMax  ||
        controller.settings.zoom < ZoomMin ) &&
        controller.settings.zoom != ZoomWidth)
    {
        controller.settings.zoom = ZoomPage;
    }        
    
    if (controller.settings.rotate != 0 &&
        controller.settings.rotate != 90 &&
        controller.settings.rotate != 180 &&
        controller.settings.rotate != 270)
    {
        controller.settings.rotate = 0;
    }        
    
    PV_LOGPRINTF("Current page %d zoom factor %lf rotate %d mode %d", 
        controller.settings.currentPage, 
        controller.settings.zoom, 
        controller.settings.rotate,         
        controller.settings.mode);
}

void CPDFApp::saveSettings()
{
    controller.settings.save(path);
}

GBool CPDFApp::open(GooString * fileName, 
                 GooString * ownerPassword, 
                 GooString * userPassword)
{
    PV_LOGPRINTF("Open document %s", fileName->getCString());
    if (errNone == controller.core->open(fileName, ownerPassword, userPassword, controller.core))
    {
        PV_LOGPRINTF("Open document %s done!\n\n", fileName->getCString());

        // initialize page list
        PDFDoc * pdfdoc = controller.core->getPDFDoc();
        controller.pageList.reset(pdfdoc->getNumPages());

        // load settings
        loadSettings(fileName);

        // try to render the pages
        controller.view->displayPage(controller.settings.currentPage);
        
        // init page bar
        controller.channels.pagebar_set_count(pdfdoc->getNumPages());
        return gTrue;
    }
    PV_ERRORPRINTF("Could not open file!");
    return gFalse;
}


void * CPDFApp::getAuthData()
{
    GooString * password = NULL;
    if (controller.view)
    {
        password = controller.view->getPassword();
        return password;
    }    
    return NULL;
}
