/* 
 * Copyright (C) 2006, iRex Technologies B.V.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * Portions of this code are taken from xpdf and are
 * Copyright 1996-2003 Glyph & Cog, LLC
 */

#include "PDFView.h"
#include "PDFViewerLog.h"
#include "goo/gtypes.h"
#include "controller.h"
#include "PageInfo.h"
#include <X11/keysym.h>
#include "liberdm/erdm.h"
#include "PDFApp.h"
#include <liberipc/eripcviewer.h>
#include <math.h>

#define IPC_BUF_MAX 1024

CPDFView::CPDFView(Widget wnd, Controller * c)
{
    drawArea = wnd; ctrl = c;
    // only focus on inputCbk
    // XtAddCallback(drawArea, XmNresizeCallback, &resizeCbk, (XtPointer)this);
    XtAddCallback(drawArea, XmNexposeCallback, &redrawCbk, (XtPointer)this);
    XtAddCallback(drawArea, XmNinputCallback, &inputCbk, (XtPointer)this);

    XtAddEventHandler(drawArea, PointerMotionMask, False, motionCbk, (XtPointer)this);
    
    // install IPC callback message handler
    int nRet = 0, fd = -1;
    erServerChannel_t channel;
    nRet = erIpcOpenServerChannel(ER_PDF_VIEWER_CHANNEL, &channel);
    ctrl->channels.erIpcChannel = channel;
    fd = erIpcGetServerFd(channel);
    XtAppAddInput(ctrl->xMgr.appContext, fd, (XtPointer)XtInputReadMask, ipcCbk, (XtPointer)this);

    // password dialog
    initPasswordDialog();
    bAuth = gTrue;

    // others
    panPt.x = panPt.y = 0;
    zoomRect.left = zoomRect.top = zoomRect.right = zoomRect.bottom = 0;
}

CPDFView::~CPDFView(void)
{
}

void CPDFView::adjustZoomRect(const int x, const int y)
{
    if (zoomRect.left > x)
    {
        zoomRect.left = x;
    }
    if (zoomRect.top > y)
    {
        zoomRect.top = y;
    }
    if (zoomRect.right < x)
    {
        zoomRect.right = x;
    }
    if (zoomRect.bottom < y)
    {
        zoomRect.bottom = y;
    }
}

GBool CPDFView::isSmallOffset(const int x, const int y)
{
    if (ctrl->channels.isPanEnable())
    {
        return (abs(panPt.x - x) < SmallOffset &&
                abs(panPt.y - y) < SmallOffset);
    }
    if (ctrl->channels.isZoomInEnable() ||
        ctrl->channels.isZoomOutEnable())
    {
        adjustZoomRect(x, y);
        return (abs(zoomRect.left - zoomRect.right) < SmallOffset &&
                abs(zoomRect.top - zoomRect.bottom) < SmallOffset);
    }
    return gTrue;
}

void CPDFView::inputCbk(Widget widget, XtPointer ptr, XtPointer callData)
{
    CPDFView *view = (CPDFView *)ptr;
    XmDrawingAreaCallbackStruct *data = (XmDrawingAreaCallbackStruct *)callData;

    switch (data->event->type) 
    {
    case ButtonPress:
        if (data->event->xbutton.button == 1)
        {
            if (view->ctrl->channels.isPanEnable())
            {
                // enable panning, record the position
                view->panPt.x = data->event->xbutton.x;
                view->panPt.y = data->event->xbutton.y;
            }
            else if (view->ctrl->channels.isZoomInEnable() ||
                     view->ctrl->channels.isZoomOutEnable())
            {
                view->zoomRect.left  = data->event->xbutton.x;
                view->zoomRect.top   = data->event->xbutton.y;
                view->zoomRect.right = data->event->xbutton.x;
                view->zoomRect.bottom= data->event->xbutton.y;
            }
        }          
        break;
    case ButtonRelease:
        if (data->event->xbutton.button == 1)
        {
            // try to do hyper link at first
            if (view->isSmallOffset(data->event->xbutton.x, data->event->xbutton.y))
            {
                PV_LOGPRINTF("Small Offset!");
                if (view->doLink(data->event->xbutton.x, data->event->xbutton.y))
                {
                    PV_LOGPRINTF("Do Link Done!");                    
                    return;
                }
                PV_LOGPRINTF("Seems no link!");                                    
            }
            if (view->ctrl->channels.isPanEnable())
            {
                view->ctrl->channels.busy_blink();
	            int deltaX = data->event->xbutton.x - view->panPt.x;
	            int deltaY = data->event->xbutton.y - view->panPt.y;
                view->ctrl->settings.screenX += deltaX;
                view->ctrl->settings.screenY += deltaY;
                view->displayPage(view->ctrl->settings.currentPage);
                return;
	        }  
	        else if (view->ctrl->channels.isZoomInEnable())
	        {
                view->ctrl->channels.busy_blink();
                view->adjustZoomRect(data->event->xbutton.x, data->event->xbutton.y);
	            view->zoomFromRect(view->zoomRect.left, 
	                               view->zoomRect.top,
	                               view->zoomRect.right,
	                               view->zoomRect.bottom);
            }	            
            else if (view->ctrl->channels.isZoomOutEnable())                    
            {
                view->ctrl->channels.busy_blink();
                view->adjustZoomRect(data->event->xbutton.x, data->event->xbutton.y);
                view->zoomToRect(view->zoomRect.left, 
                                 view->zoomRect.top,
	                             view->zoomRect.right,
	                             view->zoomRect.bottom);
            }
	    }
        break;
    case KeyPress:
        XKeyEvent * key = (XKeyEvent *)data->event;
        if (key->x_root == -1 && key->y_root == -1)
        {
            view->onPageRendered(key->x);
        }
        else
        {
            // check and issue a new request if not existing
            view->keyPressCbk(data);
        }
        break;
    }
}


void CPDFView::redrawCbk(Widget widget, XtPointer ptr, XtPointer callData)
{
    CPDFView *view = (CPDFView *)ptr;
    if (view->ctrl->settings.currentPage < 0)
    {
        view->refreshDeviceScreen(dmQFull);
    }        
    else
    {
        view->displayPage(view->ctrl->settings.currentPage);
    }
}

void CPDFView::motionCbk(Widget w, XtPointer ptr, XEvent *event, Boolean *cont)
{
    CPDFView *view = (CPDFView *)ptr;
    if (view->ctrl->channels.isZoomInEnable() || 
        view->ctrl->channels.isZoomOutEnable())
    {
        view->adjustZoomRect(event->xbutton.x, event->xbutton.y);        
    }            
}

// helper function
void CPDFView::adjustPosition(const rectangle & oldRect, double ratio)
{
    // old offset
    int offsetX = 0, offsetY = 0;
    point oldCenter, newCenter;
    oldCenter.x = (oldRect.left + oldRect.right) / 2;
    oldCenter.y = (oldRect.top + oldRect.bottom) / 2;
    offsetX = oldCenter.x - ctrl->settings.screenX;
    offsetY = oldCenter.y - ctrl->settings.screenY;

    // calculate new rect
    rectangle newRect;   
    int width  = (int)((double)(oldRect.right - oldRect.left) * ratio);
    int height = (int)((double)(oldRect.bottom - oldRect.top) * ratio);
    newRect.left = (ctrl->settings.drawAreaWidth - width) / 2;
    newRect.right = newRect.left + width;
    newRect.top = (ctrl->settings.drawAreaHeight - height) /2;
    newRect.bottom = newRect.top + height;
    
    // offset   
    newCenter.x = (newRect.left + newRect.right) / 2;
    newCenter.y = (newRect.top + newRect.bottom) / 2;
    
    ctrl->settings.screenX = newCenter.x - (int)((double)offsetX * ratio);
    ctrl->settings.screenY = newCenter.y - (int)((double)offsetY * ratio);
}

double CPDFView::calRatio(rectangle & rect, GBool bZoomin)
{
    double ratio = 1.0 , tmp = 1.0;
    int width = abs(rect.right - rect.left) + 1;    // avoid / 0
    int height = abs(rect.bottom - rect.top) + 1;   // avoid / 0
    if (bZoomin)
    {
        ratio = (double)ctrl->settings.drawAreaWidth / (double)width;
        tmp = (double)ctrl->settings.drawAreaHeight / (double)height;
    }
    else
    {
        ratio = (double)width / (double)(ctrl->settings.drawAreaWidth);
        tmp = (double)height / (double)(ctrl->settings.drawAreaHeight);
    }
    if (ratio > tmp) return tmp;
    return ratio;
}

// steps:
// 1. calcuate ratio
// 2. calcuate the new position of rectangle
// 3. adjust screenX and screenY from rectangle position and ratio
void CPDFView::zoomFromRect(int left, int top, int right, int bottom)
{
    // check rectangle and normalize rectangle
    double ratio = 1.0;
    CPageInfo * page = ctrl->pageList.getPage(ctrl->settings.currentPage);
    if (left > right) { int tmp = left; left = right; right = tmp; }
    if (top > bottom) { int tmp = top; top = bottom; bottom = tmp; }
    PV_LOGPRINTF("Zoom From rectangle (%d %d)-(%d %d)", left, top, right, bottom);
    
    rectangle rect;
    rect.left = left; rect.top = top; rect.right = right; rect.bottom = bottom;

    if (left == right || top == bottom) 
    {
        PV_LOGPRINTF("Rectangle is empty!");
        ratio = ctrl->core->getZoomMax(page->pageNumber) / page->calZoom();
        ctrl->settings.zoom = ctrl->core->getZoomMax(page->pageNumber);
        adjustPosition(rect, ratio);
    }
    else
    {        
        // calcuate zoom factor from dpi: dpi = 0.01 * zoomA * 72
        // zoom and then calcuate the new position
        ratio = calRatio(rect);
        ctrl->settings.zoom = page->calZoom() * ratio;
        if (ctrl->settings.zoom >= ctrl->core->getZoomMax(page->pageNumber))
        {
            ctrl->settings.zoom = ctrl->core->getZoomMax(page->pageNumber);
            ratio = ctrl->core->getZoomMax(page->pageNumber) / page->calZoom();
        }
        else if (ctrl->settings.zoom <= ZoomMin)
        {
            ctrl->settings.zoom = ZoomMin;
            ratio = ZoomMin / page->calZoom();
        }
        adjustPosition( rect, ratio);
    }
    displayPage(ctrl->settings.currentPage);
}

void CPDFView::zoomToRect(int left, int top, int right, int bottom)
{
    // check rectangle and normalize rectangle
    if (left > right) { int tmp = left; left = right; right = tmp; }
    if (top > bottom) { int tmp = top; top = bottom; bottom = tmp; }
    PV_LOGPRINTF("Zoom To rectangle (%d %d)-(%d %d)", left, top, right, bottom);

    double ratio = 1.0;
    CPageInfo * page = ctrl->pageList.getPage(ctrl->settings.currentPage);
    rectangle rect;
    rect.left = left; rect.top = top; rect.right = right; rect.bottom = bottom;

    if (left == right || top == bottom) 
    {
        PV_LOGPRINTF("Rectangle is empty!");
        ratio = ZoomMin / page->calZoom();
        ctrl->settings.zoom = ZoomMin;
        adjustPosition(rect, ratio);
    }
    else
    {        
        // calcuate zoom factor from dpi: dpi = 0.01 * zoomA * 72
        // zoom and then calcuate the new position
        ratio = calRatio(rect, gFalse);
        ctrl->settings.zoom = page->calZoom() * ratio;
        if (ctrl->settings.zoom >= ctrl->core->getZoomMax(page->pageNumber))
        {
            ctrl->settings.zoom = ctrl->core->getZoomMax(page->pageNumber);
            ratio = ctrl->core->getZoomMax(page->pageNumber) / page->calZoom();
        }
        else if (ctrl->settings.zoom <= ZoomMin)
        {
            ctrl->settings.zoom = ZoomMin;
            ratio = ZoomMin / page->calZoom();
        }
        adjustPosition( rect, ratio);
    }
    displayPage(ctrl->settings.currentPage);
}

void CPDFView::keyPressCbk(XmDrawingAreaCallbackStruct *data)
{
    KeySym key;
    char buf[20];
    XLookupString(&data->event->xkey, buf, sizeof(buf) - 1, &key, NULL);
    ctrl->channels.busy_blink();
    switch (key)
    {
    case XK_Home:
    case XK_KP_Home:
        // iRex. The UP Long ==> Home ==> goto toc.
        displayPage(1);
        break;
    case XK_F5:
        // changed, always quit when receive up key
        // iRex. The F5, goto first page and quit if it is in the first page
        // quit from application. generate a quit message.
        // XPDFViewer will call app to quit. temp solution.
        ctrl->app->quit();
        break;  
    case XK_Page_Up:
    case XK_KP_Page_Up:
        if (!isA4Mode())
        {
            ctrl->settings.renderDir = RenderPrev;
            displayPage(ctrl->settings.currentPage - 1);
        }
        else
        {
            A4GoPrev();
        }            
        break;
    case XK_Page_Down:
    case XK_KP_Page_Down:
        if (!isA4Mode())
        {
            ctrl->settings.renderDir = RenderNext;
            displayPage(ctrl->settings.currentPage + 1);
        }
        else
        {
            A4GoNext();
        }            
        break;
    case XK_Left:
    case XK_KP_Left:
        ctrl->channels.busy_off();
        return;
    case XK_Right:
    case XK_KP_Right:
        ctrl->channels.busy_off();
        return;
    case XK_Up:
    case XK_KP_Up:
        ctrl->channels.busy_off();
        return;
    case XK_Down:
    case XK_KP_Down:
        ctrl->channels.busy_off();
        return;
    case XK_F1: /* iRex: long pageflip forward */
        ctrl->settings.renderDir = RenderNext;
        if (ctrl->settings.currentPage + JumpWidth > ctrl->core->getPDFDoc()->getNumPages())
        {
            displayPage(ctrl->core->getPDFDoc()->getNumPages());
        }
        else
        {
            displayPage(ctrl->settings.currentPage + JumpWidth);
        }            
        break;
    case XK_F2: /* iRex: long pageflip backward */
        ctrl->settings.renderDir = RenderPrev;
        if ((ctrl->settings.currentPage - JumpWidth) < 1)
        {
            displayPage(1);
        }
        else
        {
            displayPage(ctrl->settings.currentPage - JumpWidth);
        }            
        break;
    default:
        ctrl->channels.busy_off();
        break; // no return: do not bypass keyPressCbk
    }
}

//////////////////////////////////////////////////////////////////////////
// we should check zoom max value when
// 1. issues a new request 
// 2. receives a message from render thread
// 3. query a page in page list
//////////////////////////////////////////////////////////////////////////
void CPDFView::displayPage(const int pageNumber)
{
    // check page number is valid or not
    if (pageNumber <= 0 ||
        pageNumber > ctrl->core->getPDFDoc()->getNumPages())
    {
        PV_ERRORPRINTF("Invalid page number %d!", pageNumber);
        ctrl->channels.busy_off();
        return;
    }

    // PV_LOGPRINTF("Going to display the page %d current page %d", pageNumber, ctrl->settings.currentPage);
    ctrl->settings.currentPage = pageNumber;    
    
    // query the page. checkpoint
    CPageInfo info; 
    info.pageNumber  = ctrl->settings.currentPage;
    info.pageZoom    = ctrl->core->verifyZoomMax(ctrl->settings.zoom, ctrl->settings.currentPage);
    info.rotate      = ctrl->settings.rotate;
    CPageInfo * page = ctrl->pageList.checkPage(info);
    if (page)
    {
        drawPage(page);
        issueNewRequests();
    }
    else
    {
        // checkpoint. issue a new request
        PV_LOGPRINTF("Page %d NOT ready yet!", pageNumber);
        TodoItem * item = new TodoItem;
        item->pageNumber = ctrl->settings.currentPage;
        item->rotate     = ctrl->settings.rotate;
        item->zoom       = ctrl->core->verifyZoomMax(ctrl->settings.zoom, ctrl->settings.currentPage);
        item->timeStamp  = ctrl->getTimeStamp();
        ctrl->thread.renderPageNow(item);  
        // we do not issue new requests here
    }
}

// checkpoint, issue a new request
void CPDFView::issueItem(const int pn, GBool bHead )
{
    if (pn < 1 || pn > ctrl->core->getNumPages())
    {
        PV_LOGPRINTF("Do not issue page %d", pn);
        return;
    }
    TodoItem * item = new TodoItem;
    item->pageNumber = pn;
    item->rotate     = ctrl->settings.rotate;
    item->zoom       = ctrl->core->verifyZoomMax(ctrl->settings.zoom, pn);
    item->timeStamp  = ctrl->getTimeStamp();
    ctrl->thread.addTodoItem(item, bHead);
}

void CPDFView::issuePrevNextPages(const int width)
{
    if (ctrl->settings.renderDir == RenderPrev)
    {
        for(int i = width; i >= 1; --i)
        {
            issueItem(ctrl->settings.currentPage - i);
        }            
    }   
    else if (ctrl->settings.renderDir == RenderNext)
    {
        for(int i = width; i >= 1; --i)
        {
            issueItem(ctrl->settings.currentPage + i);
        }     
    }
}

void CPDFView::issueLinks(CPageInfo *page)
{
    if (NULL == page->links) return;
    int count = page->links->getNumLinks();
    PV_DUMP("Link count %d in page %d\n", count, page->pageNumber);
    for(int i = count - 1; i >= 0; --i)
    {
        issueLink(page->links->getLink(i));
    }
}

void CPDFView::issueLink(Link * p)
{
    if (NULL == p) return;
    int pn = getLinkPage(p->getAction());
    if (pn >= 1)
    {
        PV_DUMP("Issue page %d for hyperlink\n", pn);
        issueItem(pn);
        return;
    }
    PV_LOGPRINTF("Invalid link page number %d!", pn);
}

int CPDFView::getLinkPage(LinkAction *action)
{
    if (NULL == action) return -1;
    LinkActionKind kind;
    LinkDest *dest = NULL;
    UGooString *namedDest = NULL;
  
    switch (kind = action->getKind()) 
    {
    case actionGoTo:
        dest = ((LinkGoTo *)action)->getDest();
        namedDest = ((LinkGoTo *)action)->getNamedDest();
        if (dest && dest->isPageRef()) 
        {
            Ref pageRef = dest->getPageRef();
            return ctrl->core->getPDFDoc()->findPage(pageRef.num, pageRef.gen);
        } 
        else if (namedDest)
        {
            dest = ctrl->core->getPDFDoc()->findDest(namedDest);
            if (dest)
            {
                Ref pageRef = dest->getPageRef();
                return ctrl->core->getPDFDoc()->findPage(pageRef.num, pageRef.gen);
            }                
        }
        break;
    default:
        break;        
    }
    return -1;
}

// always issue hyper link now
void CPDFView::issueNewRequests()
{
    // retrieve links
    CPageInfo * page = ctrl->pageList.getPage(ctrl->settings.currentPage);
    if (page)
    {   
        if (page->links)
        { 
            int count = page->links->getNumLinks();
            if (count <= 0)
            {
                issuePrevNextPages(RenderWidth);
            }
            else
            {                
                issuePrevNextPages(RenderWidth / 2);
                issueLinks(page);
            }
        }            
        else
        {
            issuePrevNextPages(RenderWidth);
        }
    }        
    ctrl->thread.signal();
}

void CPDFView::onPageRendered(const int pageNumber)
{
    // compare and draw, checkpoint
    if (pageNumber == ctrl->settings.currentPage)
    {
        CPageInfo info; 
        info.pageNumber = ctrl->settings.currentPage;
        info.rotate = ctrl->settings.rotate;
        info.pageZoom = ctrl->core->verifyZoomMax(ctrl->settings.zoom, ctrl->settings.currentPage);
        CPageInfo * page = ctrl->pageList.checkPage(info);
        if (page)
        {
            drawPage(page);            
            issueNewRequests();
        }            
    }
}

void CPDFView::updateToolbar(CPageInfo *page)
{
    GBool bDirty = gFalse;
    
    // pan
    int w = page->bitmap->getWidth();
    int h = page->bitmap->getHeight();
    if (w <= ctrl->settings.drawAreaWidth &&
        h <= ctrl->settings.drawAreaHeight &&
        ctrl->settings.screenX >= 0 &&
        ctrl->settings.screenY >= 0 &&
        ctrl->settings.screenX + w <= ctrl->settings.drawAreaWidth && 
        ctrl->settings.screenY + h <= ctrl->settings.drawAreaHeight)
    {
        if (ctrl->channels.getPanState() != iconState_grey)
        {
            bDirty = gTrue;
            ctrl->channels.setPanState(iconState_grey);
        }
    }
    else 
    {
        if (ctrl->channels.getPanState() == iconState_grey)
        {
            bDirty = gTrue;
            ctrl->channels.setPanState(iconState_normal);        
        }
    }

    // zoom
    double z1 = page->hDPI * 100 / DefDPI;
    double z2 = page->vDPI * 100 / DefDPI;
    double z  = (z1 > z2) ? z1 : z2;
    if (z >= ctrl->core->getZoomMax(page->pageNumber))
    {
        if (ctrl->channels.getZoomInState() != iconState_grey)
        {
            bDirty = gTrue;
            ctrl->channels.setZoomIn(iconState_grey);            
        }
    }    
    else
    {  
        if (ctrl->channels.getZoomInState() == iconState_grey ||
            ctrl->channels.getZoomInState() == iconState_selected)
        {
            bDirty = gTrue;
            ctrl->channels.setZoomIn(iconState_normal);
        }
    }
    
    if (z <= ZoomMin)
    {
        if (ctrl->channels.getZoomOutState() != iconState_grey)
        {
            bDirty = gTrue;
            ctrl->channels.setZoomOut(iconState_grey);        
        }
    }
    else
    {
        if (ctrl->channels.getZoomOutState() == iconState_grey ||
            ctrl->channels.getZoomOutState() == iconState_selected)
        {
            bDirty = gTrue;
            ctrl->channels.setZoomOut(iconState_normal);
        }
    }
    
    // rotate
    if (ctrl->settings.rotate == 0 || ctrl->settings.rotate == 180)
    {
        if (ctrl->channels.getRotateState() != Portrait_Landscape)
        {
            bDirty = gTrue;
            ctrl->channels.setRotateState(Portrait_Landscape);
        }            
    }
    else if (ctrl->settings.rotate == 90 || ctrl->settings.rotate == 270)
    {
        if (ctrl->channels.getRotateState() != Landscape_Portrait)
        {
            bDirty = gTrue;
            ctrl->channels.setRotateState(Landscape_Portrait);
        }            
    }

        
    // update
    if (bDirty)
    {
        ctrl->channels.setToolbarDirty();
        ctrl->channels.setToolbarDirtyReason(ByApp);
        ctrl->updateReq++;
        ctrl->channels.redrawToolbar();
    }
    else
    {
        ctrl->channels.setToolbarDirty(gFalse);
        ctrl->channels.setToolbarDirtyReason(ByNone);
        ctrl->updateReq = 0;
    }
}

void CPDFView::drawPage(CPageInfo * page)
{
    ctrl->channels.busy_blink();
    
    // update page time stamp
    page->timestamp = ctrl->getTimeStamp();

    // update page bar
    ctrl->channels.pagebar_set_current_page(page->pageNumber);
    
    // update toolbar 
    updateToolbar(page);
    
    // should adjust page position
    if (ctrl->settings.zoom == ZoomPage || ctrl->settings.zoom == ZoomWidth)
    {
        ctrl->settings.screenX = (ctrl->settings.drawAreaWidth - page->bitmap->getWidth()) / 2;
        ctrl->settings.screenY = (ctrl->settings.drawAreaHeight - page->bitmap->getHeight()) / 2;
    }
    ctrl->xMgr.drawImage(page->bitmap,
			 0, 0, 
			 ctrl->settings.screenX , ctrl->settings.screenY); 

    XSync(ctrl->xMgr.display, gTrue);	

    // check should we update screen right now. It could happen when toolbar isn't dirty
    // if toolbar is dirty, screen will be refreshed on receive sync message
    if (ctrl->updateReq <= 0)
    {
        refreshDeviceScreen(dmQFull);
        ctrl->updateReq = 0;
    }
    
    ctrl->pageList.dump();
}


void CPDFView::ipcCbk(XtPointer client_data, int * fd, XtInputId * id)
{
    CPDFView *viewer = (CPDFView *)client_data;
    
    // in password dialog
    if (!viewer->bAuth) return;
    
    char szBuffer[IPC_BUF_MAX]; 
    int nBuf = IPC_BUF_MAX;
    erIpcCmd_t command;
    erServerChannel_t channel = viewer->ctrl->channels.erIpcChannel;
    erIpcGetMessage(channel, szBuffer, &nBuf);

    viewer->ctrl->channels.busy_blink();
    if (vwrParseCommand(szBuffer, &command) >= 0) 
    {
        switch (command.cc) 
        {
        case ccVwrJumpToPage:
        {
            int pg = atoi(command.arg[1]);
            viewer->displayPage(pg);
            break;
        }                
        case ccVwrToolbar:
        {
            int iconID = atoi(command.arg[0]);
            int state  = atoi(command.arg[1]);
            if (viewer->ctrl->channels.onReceiveToolbarMsg(iconID, state))
            {
                viewer->ctrl->channels.setToolbarDirtyReason(ByUserAction);
                viewer->ctrl->channels.redrawToolbar();            
            }
            else if (iconID != iconID_landscape_portrait &&
                     iconID != iconID_portrait_landscape && 
                     iconID != iconID_zoomfit)
            {
                // hit grey icons
                viewer->ctrl->channels.busy_off();
            }

            if (iconID == iconID_zoomfit)
            {
                // update settings, reset the screen position
                viewer->ctrl->settings.screenX = 0;
                viewer->ctrl->settings.screenY = 0;                
                viewer->ctrl->settings.zoom = ZoomPage;
                viewer->displayPage(viewer->ctrl->settings.currentPage);
            }
            else if (iconID == iconID_portrait_landscape)
            {
                // turn left or turn right. it seems something wrong with sync message
                // which can not ensure toolbar has been redrawn
                // so i have to draw it here, temp solution.
                if (viewer->ctrl->channels.getRotateState() == Landscape_Portrait)
                {
                    viewer->rotateCurrentPage(+90);
                    viewer->ctrl->channels.setRotateState(Portrait_Landscape);
                    viewer->ctrl->channels.setToolbarDirty();
                    viewer->ctrl->channels.redrawToolbar(gFalse);
                    viewer->ctrl->channels.setToolbarDirty(gFalse);
                }
                else if (viewer->ctrl->channels.getRotateState() == Portrait_Landscape)                    
                {
                    viewer->rotateCurrentPage(-90);
                    viewer->ctrl->channels.setRotateState(Landscape_Portrait);                    
                    viewer->ctrl->channels.setToolbarDirty();
                    viewer->ctrl->channels.redrawToolbar(gFalse);
                    viewer->ctrl->channels.setToolbarDirty(gFalse);
                }
            }
            break;
        }
        case ccVwrToolbarSync:
        {
            if (viewer->ctrl->channels.getToolbarDirtyReason() == ByUserAction)
            {
                viewer->refreshDeviceScreen(dmQTyping);
                viewer->ctrl->channels.setToolbarDirty(gFalse);
                viewer->ctrl->channels.setToolbarDirtyReason(ByNone);
            }
            // happens when a new page is displayed. ByApp or ByNone
            else if (viewer->ctrl->channels.getToolbarDirtyReason())
            {
                --viewer->ctrl->updateReq;
                if (viewer->ctrl->updateReq <= 0)
                {
                    viewer->refreshDeviceScreen(dmQFull);
                }
                viewer->ctrl->channels.setToolbarDirty(gFalse);
                viewer->ctrl->channels.setToolbarDirtyReason(ByNone);
            }
            break;
        }            
        default:
            break;
        }
    }

    return;
}

// rotate current page
// if zoom factor is not zoomPage, we should adjust current screenX and screenY
// so that we can make it in the same position
// steps:
// 1. get center point of intersection with screen
// 2. calculate the new screenx and screeny from center point and angle
// input parameter angle is a clock wise based incremental value
void CPDFView::rotateCurrentPage(const int angle)
{
    // get page info
    CPageInfo * page = ctrl->pageList.getPage(ctrl->settings.currentPage);

    // calculate intersection
    rectangle rect; rect.left = 0; rect.top = 0; 
    rect.right = ctrl->settings.drawAreaWidth;
    rect.bottom = ctrl->settings.drawAreaHeight;
    if (ctrl->settings.screenX > 0) rect.left = ctrl->settings.screenX;
    if (ctrl->settings.screenY > 0) rect.top = ctrl->settings.screenY;
    if (rect.right > ctrl->settings.screenX + page->bitmap->getWidth())
    {
        rect.right = ctrl->settings.screenX + page->bitmap->getWidth();
    }
    if (rect.bottom > ctrl->settings.screenY + page->bitmap->getHeight())
    {
        rect.bottom = ctrl->settings.screenY + page->bitmap->getHeight();
    }
    point center;
    center.x = (rect.left + rect.right) / 2;
    center.y = (rect.top + rect.bottom) / 2;
    int distX = 0, distY = 0;
    
    // calculate the new position 
    if (angle == 90)
    {
        distX = center.x - ctrl->settings.screenX;
        distY = page->bitmap->getHeight() - center.y + ctrl->settings.screenY;
        ctrl->settings.screenX = center.x - distY;
        ctrl->settings.screenY = center.y - distX;
    }
    else if (angle == -90)
    {
        distX = page->bitmap->getWidth() - center.x + ctrl->settings.screenX;
        distY = center.y - ctrl->settings.screenY;
        ctrl->settings.screenX = center.x - distY;
        ctrl->settings.screenY = center.y - distX;
    }
    ctrl->settings.rotate += angle;
    if (ctrl->settings.rotate < 0)
    {
        ctrl->settings.rotate += 360;
    }
    else if (ctrl->settings.rotate >= 360)
    {
        ctrl->settings.rotate -= 360;
    }
    displayPage(ctrl->settings.currentPage);
}

void CPDFView::refreshDeviceScreen(eDmQuality quality)
{
    dmDisplay(dmCmdPriorUrgent, quality);
    ctrl->channels.busy_off();
}

void CPDFView::cvtDevToUser(CPageInfo *page, const int x, const int y, double & ux, double & uy)
{
    ux = page->ictm[0] * (x - ctrl->settings.screenX) + 
         page->ictm[2] * (y - ctrl->settings.screenY) + page->ictm[4];
    uy = page->ictm[1] * (x - ctrl->settings.screenX) + 
         page->ictm[3] * (y - ctrl->settings.screenY) + page->ictm[5];    
}

GBool CPDFView::doLink(int x, int y)
{
    CPageInfo * page = ctrl->pageList.getPage(ctrl->settings.currentPage);
    if (NULL == page || NULL == page->links) return gFalse;
    double ux = 0.0, uy = 0.0;
    cvtDevToUser(page, x, y, ux, uy);
    LinkAction * action = page->links->find(ux, uy);
    if (NULL == action) return gFalse;
    return doAction(action);
}

GBool CPDFView::doAction(LinkAction * action)
{
    PV_LOGPRINTF("Do some action!");
    LinkActionKind kind;
    LinkDest *dest = NULL;
    UGooString *namedDest = NULL;
  
    switch (kind = action->getKind()) 
    {
    case actionGoTo:
        ctrl->channels.busy_blink();
        PV_LOGPRINTF("actionGoTo");
        dest = ((LinkGoTo *)action)->getDest();
        namedDest = ((LinkGoTo *)action)->getNamedDest();
        if (dest && dest->isPageRef()) 
        {
            Ref pageRef = dest->getPageRef();
            int pageNumber = ctrl->core->getPDFDoc()->findPage(pageRef.num, pageRef.gen);
            PV_LOGPRINTF("Goto Page %d", pageNumber);
            displayPage(pageNumber);
            return gTrue;
        } 
        else if (namedDest)
        {
            dest = ctrl->core->getPDFDoc()->findDest(namedDest);
            if (dest)
            {
                Ref pageRef = dest->getPageRef();
                int pageNumber = ctrl->core->getPDFDoc()->findPage(pageRef.num, pageRef.gen);
                PV_LOGPRINTF("Goto Page %d", pageNumber);
                displayPage(pageNumber);
                return gTrue;
            }                
        }
        break;
    case actionLaunch:
        PV_LOGPRINTF("actionLaunch!");
        break;
                
    case  actionNamed:
        PV_LOGPRINTF("actionNamed!");     
        break;
    case  actionURI:
        PV_LOGPRINTF("actionURI!");     
        break;
    case actionMovie:
        PV_LOGPRINTF("actionMovie!");     
        break;
    default:
        PV_LOGPRINTF("unknown!");        
        break;
    }
    return gFalse;
/*

  // Named action
  case actionNamed:
    actionName = ((LinkNamed *)action)->getName();
    if (!actionName->cmp("NextPage")) {
      gotoNextPage(1, gTrue);
    } else if (!actionName->cmp("PrevPage")) {
      gotoPrevPage(1, gTrue, gFalse);
    } else if (!actionName->cmp("FirstPage")) {
      if (topPage != 1) {
	displayPage(1, zoom, rotate, gTrue, gTrue);
      }
    } else if (!actionName->cmp("LastPage")) {
      if (topPage != doc->getNumPages()) {
	displayPage(doc->getNumPages(), zoom, rotate, gTrue, gTrue);
      }
    } else if (!actionName->cmp("GoBack")) {
      goBackward();
    } else if (!actionName->cmp("GoForward")) {
      goForward();
    } else if (!actionName->cmp("Quit")) {
      if (actionCbk) {
	(*actionCbk)(actionCbkData, "Quit");
      }
    } else {
      error(-1, "Unknown named action: '%s'", actionName->getCString());
    }
    break;

  // Movie action
  case actionMovie:
    if (!(cmd = globalParams->getMovieCommand())) {
      error(-1, "No movieCommand defined in config file");
      break;
    }
    if (((LinkMovie *)action)->hasAnnotRef()) {
      doc->getXRef()->fetch(((LinkMovie *)action)->getAnnotRef()->num,
			    ((LinkMovie *)action)->getAnnotRef()->gen,
			    &movieAnnot);
    } else {
      //~ need to use the correct page num here
      doc->getCatalog()->getPage(topPage)->getAnnots(&obj1);
      if (obj1.isArray()) {
	for (i = 0; i < obj1.arrayGetLength(); ++i) {
	  if (obj1.arrayGet(i, &movieAnnot)->isDict()) {
	    if (movieAnnot.dictLookup("Subtype", &obj2)->isName("Movie")) {
	      obj2.free();
	      break;
	    }
	    obj2.free();
	  }
	  movieAnnot.free();
	}
	obj1.free();
      }
    }
    if (movieAnnot.isDict()) {
      if (movieAnnot.dictLookup("Movie", &obj1)->isDict()) {
	if (obj1.dictLookup("F", &obj2)) {
	  if ((fileName = LinkAction::getFileSpecName(&obj2))) {
	    if (!isAbsolutePath(fileName->getCString())) {
	      fileName2 = appendToPath(
			      grabPath(doc->getFileName()->getCString()),
			      fileName->getCString());
	      delete fileName;
	      fileName = fileName2;
	    }
	    runCommand(cmd, fileName);
	    delete fileName;
	  }
	  obj2.free();
	}
	obj1.free();
      }
    }
    movieAnnot.free();
    break;

  // unknown action type
  case actionUnknown:
    error(-1, "Unknown link action type: '%s'",
	  ((LinkUnknown *)action)->getAction()->getCString());
    break;
  }
  */
}

//////////////////////////////////////////////////////////////////////////
// password authentication
void CPDFView::initPasswordDialog()
{
    Widget row, label, okBtn, cancelBtn;
    Arg args[20];
    int n;
    XmString s;

    //----- dialog
    n = 0;
    s = XmStringCreateLocalized(AppName ": Password");
    XtSetArg(args[n], XmNdialogTitle, s); ++n;
    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL); ++n;
    passwordDialog = XmCreateFormDialog(drawArea, "passwordDialog", args, n);
    XmStringFree(s);

    //----- message
    n = 0;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); ++n;
    XtSetArg(args[n], XmNtopOffset, 4); ++n;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); ++n;
    XtSetArg(args[n], XmNleftOffset, 4); ++n;
    s = XmStringCreateLocalized("This document requires a password.");
    XtSetArg(args[n], XmNlabelString, s); ++n;
    label = XmCreateLabel(passwordDialog, "msg", args, n);
    XmStringFree(s);
    XtManageChild(label);

    //----- label and password entry
    n = 0;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); ++n;
    XtSetArg(args[n], XmNtopWidget, label); ++n;
    XtSetArg(args[n], XmNtopOffset, 4); ++n;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); ++n;
    XtSetArg(args[n], XmNleftOffset, 4); ++n;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); ++n;
    XtSetArg(args[n], XmNleftOffset, 4); ++n;
    XtSetArg(args[n], XmNorientation, XmHORIZONTAL); ++n;
    XtSetArg(args[n], XmNpacking, XmPACK_TIGHT); ++n;
    row = XmCreateRowColumn(passwordDialog, "row", args, n);
    XtManageChild(row);
    n = 0;
    s = XmStringCreateLocalized("Password: ");
    XtSetArg(args[n], XmNlabelString, s); ++n;
    label = XmCreateLabel(row, "label", args, n);
    XmStringFree(s);
    XtManageChild(label);
    n = 0;
    XtSetArg(args[n], XmNcolumns, 16); ++n;
    passwordText = XmCreateTextField(row, "text", args, n);
    XtManageChild(passwordText);
    XtAddCallback(passwordText, XmNmodifyVerifyCallback,
		&passwordTextVerifyCbk, this);

    //----- "Ok" and "Cancel" buttons
    n = 0;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); ++n;
    XtSetArg(args[n], XmNtopWidget, row); ++n;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); ++n;
    XtSetArg(args[n], XmNleftOffset, 4); ++n;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); ++n;
    XtSetArg(args[n], XmNbottomOffset, 4); ++n;
    XtSetArg(args[n], XmNnavigationType, XmEXCLUSIVE_TAB_GROUP); ++n;
    okBtn = XmCreatePushButton(passwordDialog, "Ok", args, n);
    XtManageChild(okBtn);
    XtAddCallback(okBtn, XmNactivateCallback,
		&passwordOkCbk, (XtPointer)this);
    n = 0;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); ++n;
    XtSetArg(args[n], XmNtopWidget, row); ++n;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); ++n;
    XtSetArg(args[n], XmNrightOffset, 4); ++n;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); ++n;
    XtSetArg(args[n], XmNbottomOffset, 4); ++n;
    XtSetArg(args[n], XmNnavigationType, XmEXCLUSIVE_TAB_GROUP); ++n;
    cancelBtn = XmCreatePushButton(passwordDialog, "Cancel", args, n);
    XtManageChild(cancelBtn);
    XtAddCallback(cancelBtn, XmNactivateCallback,
		&passwordCancelCbk, (XtPointer)this);
    n = 0;
    XtSetArg(args[n], XmNdefaultButton, okBtn); ++n;
    XtSetArg(args[n], XmNcancelButton, cancelBtn); ++n;
#if XmVersion > 1001
    XtSetArg(args[n], XmNinitialFocus, passwordText); ++n;
#endif
    XtSetValues(passwordDialog, args, n);
}

void CPDFView::passwordTextVerifyCbk(Widget widget, XtPointer ptr,
				     XtPointer callData)
{
    CPDFView *view = (CPDFView *)ptr;
    XmTextVerifyCallbackStruct *data = (XmTextVerifyCallbackStruct *)callData;
    int i, n;

    i = (int)data->startPos;
    n = (int)data->endPos - i;
    if (i > view->password->getLength()) 
    {
        i = view->password->getLength();
    }
    if (i + n > view->password->getLength()) 
    {
        n = view->password->getLength() - i;
    }
    view->password->del(i, n);
    view->password->insert(i, data->text->ptr, data->text->length);

    for (i = 0; i < data->text->length; ++i) 
    {
        data->text->ptr[i] = '*';
    }
    data->doit = True;
}

void CPDFView::passwordOkCbk(Widget widget, XtPointer ptr,
			     XtPointer callData) 
{
    CPDFView *view = (CPDFView *)ptr;
    view->dialogDone = 1;
}

void CPDFView::passwordCancelCbk(Widget widget, XtPointer ptr,
				 XtPointer callData) 
{
    CPDFView *view = (CPDFView *)ptr;
    view->dialogDone = -1;
}

GooString *CPDFView::getPassword() 
{
    XtAppContext appContext;
    XEvent event;

    // NB: set <password> before calling XmTextFieldSetString, because
    // SetString will trigger a call to passwordTextVerifyCbk, which
    // expects <password> to be valid
    password = new GooString();
    bAuth = gFalse;
    XmTextFieldSetString(passwordText, "");
    XtManageChild(passwordDialog);

    appContext = XtWidgetToApplicationContext(passwordDialog);
    dialogDone = 0;
    do 
    {
        XtAppNextEvent(appContext, &event);
        XtDispatchEvent(&event);

        if (event.type == KeyPress)
        {
            XSync(ctrl->xMgr.display, TRUE);
            refreshDeviceScreen(dmQTyping);
        }
        if (event.type == Expose)
        {
            // not correct, but works    
            ctrl->channels.popupKeyboard();
            refreshDeviceScreen(dmQFull);
        }
    }
    while (!dialogDone);
    bAuth = gTrue;
    XtUnmanageChild(passwordDialog);
    ctrl->channels.popupKeyboard(gFalse);

    if (dialogDone < 0) 
    {
        delete password; password = NULL;
        return NULL;
    }
    return password;
}

//////////////////////////////////////////////////////////////////////////
// A4 Reading
//////////////////////////////////////////////////////////////////////////
#define A4Width     1683
#define A4Height    1190

void CPDFView::startA4Reading()
{
    int type = ctrl->core->getPageType(ctrl->settings.currentPage);
    if (40 != type && 41 != type)
    {
        PV_LOGPRINTF("It's not a A4 or letter page %d!", type);
        ctrl->channels.busy_off();
        return;
    }
    
    // calculate layout
    static const double A4Zoom      = 190.00f;
    static const int    A4Rotate    = 90;
    ctrl->settings.screenX = ctrl->settings.drawAreaWidth - A4Width;
    ctrl->settings.screenY = 0;
    ctrl->settings.rotate  = A4Rotate;
    ctrl->settings.zoom    = A4Zoom;
    ctrl->settings.mode    = ModeA4;
    displayPage(ctrl->settings.currentPage);
}

GBool CPDFView::isA4Mode()
{
    if (ctrl && ctrl->settings.mode == ModeA4)
    {
        return gTrue;
    }
    return gFalse;
}


void CPDFView::A4GoPrev()
{
    if (ctrl->settings.screenX >= (2 * ctrl->settings.drawAreaWidth - A4Width))
    {
        ctrl->settings.screenX -= ctrl->settings.drawAreaWidth;
        displayPage(ctrl->settings.currentPage);
    }
    else if (ctrl->settings.screenX <= ( ctrl->settings.drawAreaWidth - A4Width))
    {
        if (ctrl->settings.currentPage > 1)
        {
            ctrl->settings.screenX = ctrl->settings.drawAreaWidth - A4Width;
            displayPage(ctrl->settings.currentPage - 1);
        }            
    }
}

void CPDFView::A4GoNext()
{
    if (ctrl->settings.screenX <= (ctrl->settings.drawAreaWidth - A4Width))
    {
        ctrl->settings.screenX += ctrl->settings.drawAreaWidth;
        displayPage(ctrl->settings.currentPage);
    }
    else if (ctrl->settings.screenX >= (2 * ctrl->settings.drawAreaWidth - A4Width))
    {
        if (ctrl->settings.currentPage < ctrl->core->getPDFDoc()->getNumPages())
        {
            ctrl->settings.screenX = ctrl->settings.drawAreaWidth - A4Width;
            displayPage(ctrl->settings.currentPage + 1);
        }            
    }
}

