/* 
 * Copyright (C) 2006, iRex Technologies B.V.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * Portions of this code are taken from xpdf and are
 * Copyright 1996-2003 Glyph & Cog, LLC
 */

#include "XMgr.h"
#include "goo/gmem.h"
#include "PDFViewerLog.h"
#include <liberdm/display.h>
#define defaultRGBCube  5

//////////////////////////////////////////////////////////////////////////
static String fallbackResources[] = {
  NULL
};

static XrmOptionDescRec xOpts[] = {
};

#define nXOpts (sizeof(xOpts) / sizeof(XrmOptionDescRec))

struct XPDFAppResources 
{
    String geometry;
    String title;
    Bool installCmap;
    int rgbCubeSize;
    Bool reverseVideo;
    String paperColor;
    String matteColor;
    String initialZoom;
    Bool viKeys;
};

#define nXResources (sizeof(xResources) / sizeof(XtResource))

// these macro defines conflict with xpdf's Object class
#ifdef LESSTIF_VERSION
#undef XtDisplay
#undef XtScreen
#undef XtWindow
#undef XtParent
#undef XtIsRealized
#endif

XMgr::XMgr(void)
: display(NULL)
, screenNum(0)
, drawAreaGC(NULL)
, visual(NULL)
, rgbCubeSize(5)
, depth(8)
, mattePixel(Gulong(0x00ffffff))
, shmInfo(NULL)
, shmImage(NULL)
{
}

XMgr::~XMgr(void)
{
    XtDestroyWidget(win);

    // Tear down shared memory connection to X
    if (shmInfo) {
      // Tell X to detach from shared  memory buffer
      XShmDetach(display, shmInfo);

      // Detach us from shared memory buffer
      shmdt(shmInfo->shmaddr);

      // Tell kernel to dispose of shared memory buffer
      shmctl(shmInfo->shmid, IPC_RMID, 0);

      // Tear down X shared memory image
      if (shmImage) {
	shmImage->data=0;
	XDestroyImage(shmImage);
	shmImage=0;
      }
    }
    delete shmInfo;
}

void XMgr::setupX(int * argc, char * argv[])
{
    int n = 0;
    Arg args[20];

    // app shell
    appShell  = XtAppInitialize(&appContext, AppName, xOpts, nXOpts, 
                    argc, argv, fallbackResources, NULL, 0);
    display   = XtDisplay(appShell);
    screenNum = XScreenNumberOfScreen(XtScreen(appShell));
    visual    = DefaultVisual(display, screenNum);
    
    // initialize app shell 
    n = 0;
    XtSetArg(args[n], XmNwidth, SCREEN_WIDTH); ++n;
    XtSetArg(args[n], XmNheight, CLIENT_AREA); ++n;
    XtSetValues(appShell, args, n);

    // main window
    n = 0;
    XtSetArg(args[n], XmNminWidth, SCREEN_WIDTH); ++n;
    XtSetArg(args[n], XmNminHeight, CLIENT_AREA); ++n;
    XtSetArg(args[n], XmNbaseWidth, SCREEN_WIDTH); ++n;
    XtSetArg(args[n], XmNbaseHeight, CLIENT_AREA); ++n;
    XtSetArg(args[n], XmNdeleteResponse, XmDO_NOTHING); ++n;
    XtSetArg(args[n], XmNgeometry, "+0+0"); ++n;

    // should be win, the name should be same with registry
    win = XtCreatePopupShell("win", topLevelShellWidgetClass, 
            appShell, args, n);

    // draw area
    n = 0;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_ANY); ++n;
    XtSetArg(args[n], XmNwidth, SCREEN_WIDTH); ++n;
    XtSetArg(args[n], XmNheight, CLIENT_AREA); ++n;
    XtSetArg(args[n], XmNbackground, 0x0ff); ++n;
    drawArea = XmCreateDrawingArea(win, "drawArea", args, n);
    XtManageChild(drawArea);   
    XtOverrideTranslations(drawArea, XtParseTranslationTable(
      "<Btn1Down>:DrawingAreaInput()\n"
      "<Btn1Up>:DrawingAreaInput()\n"
      "<Btn1Motion>:DrawingAreaInput()\n"
      "<Motion>:DrawingAreaInput()"));

    // popup window
    XtPopup(win, XtGrabNone); 

    // Setup out shmInfo
    shmInfo = new XShmSegmentInfo;
    shmInfo->shmid = shmget (IPC_PRIVATE, SCREEN_WIDTH * SCREEN_HEIGHT, IPC_CREAT|0777);
    shmInfo->shmaddr = (char *)shmat (shmInfo->shmid, 0, 0);
    shmInfo->readOnly = gFalse;

    // Create image wrapper on X server for our shared memory
    shmImage = XShmCreateImage(display,
			       visual,
			       8,		// Depth of 8
			       ZPixmap, 
			       shmInfo->shmaddr,
			       shmInfo,
			       SCREEN_WIDTH,
			       CLIENT_AREA);

    // Attach X server to our shared memory buffer
    XShmAttach (display, shmInfo);
}

void XMgr::drawImage(SplashBitmap * bmp, 
		     int xSrc , int ySrc , 
		     int xDest, int yDest) 
{
    Window drawAreaWin;
    XGCValues gcValues;

    // create a GC for the drawing area
    drawAreaWin = XtWindow(drawArea);
    if (!drawAreaGC) 
    {
        gcValues.foreground = mattePixel;
        drawAreaGC = XCreateGC(display, drawAreaWin, GCForeground, &gcValues);
    }
    else
    {
      //XSetForeground(display, drawAreaGC, mattePixel);
    }

    // Get our source image width and calculate delta frem screen width
    int srcWidth = bmp->getWidth();
    int deltaX = SCREEN_WIDTH - srcWidth;
    if (deltaX < 0) {
      // Source too wide, trim to screen width and set deltaX to 0
      srcWidth = SCREEN_WIDTH;
    }

    // Get our source image height and calculate delta frem screen height
    int srcHeight = bmp->getHeight();
    int deltaY = CLIENT_AREA - srcHeight;
    if (deltaY < 0) {
      // Source too tall, trim to screen height and set deltaY to 0
      srcHeight = CLIENT_AREA;
    }

    // Now move image data into shared memory buffer
    char *src = (char*)bmp->getDataPtr();
    char *dest = shmInfo->shmaddr;
    for (int i = 0; i < srcHeight; i++) {
      // Copy source line of data
      memcpy(dest, src, srcWidth);
      src += srcWidth;
      dest += srcWidth;

      // Fill rest of line with white
      if (deltaX > 0) {
	memset(dest, 255, deltaX);
	dest += deltaX;
      } else {
	// Skip extra source line material
	src -= deltaX;
      }
    }

    // Fill rest of page with white
    if (deltaY > 0) {
      memset(dest, 255, deltaY * SCREEN_WIDTH);
    }

    // Tell X to put it up
    XShmPutImage(display, drawAreaWin, drawAreaGC,
		 shmImage,
		 0, 0, 
		 0, 0, 
		 SCREEN_WIDTH, CLIENT_AREA,
		 gFalse);
            
}
