/* 
 * Copyright (C) 2006, iRex Technologies B.V.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "channels.h"
#include "liberipc/eripcviewer.h"

CChannels::CChannels()
{
    initialize();
}

CChannels::~CChannels()
{
}

void CChannels::initialize()
{
    // busy indicator
    erIpcStartClient(ER_BUSYD_CHANNEL, &erbusyChannel);

    // pagebar
    erIpcStartClient(ER_PAGEBAR_CHANNEL, &erpagebarChannel);
    
    // toolbar 
    erIpcStartClient(ER_TOOLBAR_CHANNEL, &ertoolbarChannel);
    tbSelectIconSet(ertoolbarChannel, ER_PDF_VIEWER_UA_ID);
    tbClearIconSet(ertoolbarChannel, ER_PDF_VIEWER_UA_ID);
    
    // trashcan
    tbAppendPlatformIcon(  ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_trashcan, -1);
    tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_trashcan, iconState_grey );
    tbAppendPlatformIcon(  ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_keyboard, -1);
    tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_keyboard, iconState_normal );
    
    // zoom in/out, pan, rotation. init the state here
    tbAppendPlatformIcon(  ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_zoomin, ccVwrToolbar);
    tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_zoomin, iconState_normal);
    // temp. 
    // tbAppendPlatformIcon(  ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_zoomout, ccVwrToolbar);
    // tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_zoomout, iconState_normal );
    tbAppendPlatformIcon(  ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_zoomfit, ccVwrToolbar);
    tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_zoomfit, iconState_normal );
    tbAppendPlatformIcon(  ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_pan, ccVwrToolbar);
    tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_pan, iconState_normal );

    // rotate
    tbAppendPlatformIcon(  ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_portrait_landscape, ccVwrToolbar);
    tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_portrait_landscape, iconState_normal );
    nPanState = nZoominState = nZoomoutState = iconState_normal;
    nRotateState = Portrait_Landscape;
}

//////////////////////////////////////////////////////////////////////////
// busy indicator
void CChannels::busy_blink()
{
    busySetBusy(erbusyChannel, ccBusyState_Blink);
}
 
void CChannels::busy_on()
{
    busySetBusy(erbusyChannel, ccBusyState_On);
}

void CChannels::busy_off()
{
    busySetBusy(erbusyChannel, ccBusyState_Off);
}

//////////////////////////////////////////////////////////////////////////
// page bar
void CChannels::pagebar_set_count(const int count)
{
    pbReset(erpagebarChannel, ER_PDF_VIEWER_UA_ID);
    pbSetPageCount(erpagebarChannel, ER_PDF_VIEWER_UA_ID, count);
    pbRedraw(erpagebarChannel, ER_PDF_VIEWER_UA_ID);
}

void CChannels::pagebar_set_current_page(const int number)
{
    pbSetCurrentPage(erpagebarChannel, ER_PDF_VIEWER_UA_ID, number);
    pbRedraw(erpagebarChannel, ER_PDF_VIEWER_UA_ID);
}

// invoked when user clicks the icons
GBool CChannels::onReceiveToolbarMsg(const int iconID, const int state)
{
    if (iconID == iconID_zoomin)
    {
        if (iconState_selected == nZoominState)
        {
            setToolbarDirty();
            setZoomIn(iconState_normal);
        }
        else if (iconState_normal == nZoominState)
        {
            setToolbarDirty();
            setZoomIn(iconState_selected);
            if (isZoomOutEnable())
            {
                setZoomOut(iconState_normal);
            }
            if (isPanEnable())
            {
                setPanState(iconState_normal);
            }  
        }            
    }
    else if (iconID == iconID_zoomout)
    {
        if (iconState_selected == nZoomoutState)
        {
            setToolbarDirty();
            setZoomOut(iconState_normal);
        }
        else if (iconState_normal == nZoomoutState)
        {
            setToolbarDirty();
            setZoomOut(iconState_selected); 
            if (isZoomInEnable())
            {
                setZoomIn(iconState_normal);   
            }  
            if (isPanEnable())
            {
                setPanState(iconState_normal);
            }                          
        }
    }
    else if (iconID == iconID_pan)
    {
        if (iconState_normal == nPanState)
        {
            setToolbarDirty();
            setPanState(iconState_selected);
            if (isZoomInEnable())
            {
                setZoomIn(iconState_normal);   
            }  
            if (isZoomOutEnable())
            {
                setZoomOut(iconState_normal);   
            }  
        }
        else if (iconState_selected == nPanState)
        {
            setToolbarDirty();
            setPanState(iconState_normal);
        }            
    }
    return isToolbarDirty();
}
 
void CChannels::redrawToolbar(GBool bSync)
{
    if (isToolbarDirty())
    {
        tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_pan, nPanState);
        tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_zoomin, nZoominState);
        tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_zoomout, nZoomoutState);
        
        if (Portrait_Landscape == nRotateState)
        {
            tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_portrait_landscape, iconState_normal);
        }
        else if (Landscape_Portrait == nRotateState)
        {
            tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_portrait_landscape, 3);
        }
        if (bSync)
        {
            tbSynchronise(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, ccVwrToolbarSync);
        }            
        setToolbarDirty(gFalse);
    }
}

void CChannels::popupKeyboard(GBool bPopup)
{   
    if (bPopup)
    {
        tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_keyboard, iconState_selected);
    }        
    else
    {
        tbSetStatePlatformIcon(ertoolbarChannel, ER_PDF_VIEWER_UA_ID, iconID_keyboard, iconState_normal);
    }
}
