/* 
 * Copyright (C) 2006, iRex Technologies B.V.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "controller.h"
#include <libermanifest/ermanifest.h>

void Settings::load(const char * pathName)
{
    erManifest manifest;
    if (RET_OK == ermXmlOpen(pathName, &manifest))
    {
        // usually the manifest file is existing
        ermXmlGetInt(&manifest, "/package/last-location/pagenumber", &currentPage);
    
        // zoom
        static const int MAX = 20;
        char tmp[MAX] = {0};
        ermXmlGetString(&manifest, "/package/viewer-settings/zoomfactor", tmp, MAX);
        
        if (0 == strcasecmp(tmp, "zoomPage"))
        {
            zoom = ZoomPage;
        }
        else if (0 == strcasecmp(tmp, "zoomWidth"))
        {
            zoom = ZoomWidth;
        }
        else
        {
            sscanf(tmp, "%lf", &zoom);
        }
        
        // rotation & position
        ermXmlGetInt(&manifest, "/package/viewer-settings/rotation", &rotate);
        ermXmlGetInt(&manifest, "/package/viewer-settings/positionx", &screenX);
        ermXmlGetInt(&manifest, "/package/viewer-settings/positiony", &screenY);
        
        // s.dump();
    }    
    
    ermXmlClose(&manifest);    
}

void Settings::save(const char *pathName)
{
    erManifest manifest;
    if (RET_OK == ermXmlOpen(pathName, &manifest))
    {
        // usually the manifest file is existing
        
        // dump for debug
        // s.dump();
        
        // page
        if (RET_OK != ermXmlExist(&manifest, "/package/last-location/pagenumber"))
        {
            ermXmlNewString(&manifest, "/package", "last-location", "");            
            ermXmlNewString(&manifest, "/package/last-location", "pagenumber", "");            
        }
        ermXmlSetInt(&manifest, "/package/last-location/pagenumber", currentPage);                    

        // zoom 
        if (RET_OK != ermXmlExist(&manifest, "/package/viewer-settings/zoomfactor"))
        {
            ermXmlNewString(&manifest, "/package", "viewer-settings", "");            
            ermXmlNewString(&manifest, "/package/viewer-settings", "zoomfactor", "");            
        }
        if (zoom == ZoomPage)
        {
            ermXmlSetString(&manifest, "/package/viewer-settings/zoomfactor", "zoomPage", strlen("zoomPage"));                    
        }
        else if (zoom == ZoomWidth)
        {
            ermXmlSetString(&manifest, "/package/viewer-settings/zoomfactor", "zoomWidth", strlen("zoomWidth"));                    
        }            
        else 
        {
            char tmp[20] = {0};
            snprintf(tmp, 20, "%lf", zoom);
            ermXmlSetString(&manifest, "/package/viewer-settings/zoomfactor", tmp, strlen(tmp));                    
        }
        
        // rotation
        if (RET_OK != ermXmlExist(&manifest, "/package/viewer-settings/rotation"))
        {
            ermXmlNewString(&manifest, "/package/viewer-settings", "rotation", "");            
        }
        ermXmlSetInt(&manifest, "/package/viewer-settings/rotation", rotate);                    

        // position x
        if (RET_OK != ermXmlExist(&manifest, "/package/viewer-settings/positionx"))
        {
            ermXmlNewString(&manifest, "/package/viewer-settings", "positionx", "");            
        }
        ermXmlSetInt(&manifest, "/package/viewer-settings/positionx", screenX);                    

        // position y
        if (RET_OK != ermXmlExist(&manifest, "/package/viewer-settings/positiony"))
        {
            ermXmlNewString(&manifest, "/package/viewer-settings", "positiony", "");            
        }
        ermXmlSetInt(&manifest, "/package/viewer-settings/positiony", screenY);                    

        ermXmlSaveAs(pathName, &manifest);
    }     
    
}
